/*         ______   ___    ___
 *        /\  _  \ /\_ \  /\_ \
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *
 *      Inline functions (MSVC style 386 asm).
 *
 *      By Stefan Schimanski.
 *
 *      See readme.txt for copyright information.
 */


#if (!defined ALLEGRO_MSVC) || (!defined ALLEGRO_I386)
   #error bad include
#endif



/* _default_ds:
 *  Return a copy of the current %ds selector.
 */
__inline int _default_ds(void)
{
   __asm {
      mov eax, 0
      mov ax, ds 
   }
}



/* bmp_write_line:
 *  Bank switch function.
 */
__inline unsigned long bmp_write_line(BITMAP *bmp, int line)
{
   int write_bank = (int)bmp->write_bank;
   int result;

   _asm { 
      mov edx, bmp
      mov ecx, write_bank
      mov eax, line
      call ecx
      mov result, eax
   }

   return result;
}



/* bmp_read_line:
 *  Bank switch function.
 */
__inline unsigned long bmp_read_line(BITMAP *bmp, int line)
{
   int read_bank = (int)bmp->read_bank;
   int result;

   _asm {
      mov edx, bmp
      mov ecx, read_bank
      mov eax, line
      call ecx
      mov result, eax
   }

   return result;
}



/* bmp_unwrite_line:
 *  Terminate bank switch function.
 */
__inline void bmp_unwrite_line(BITMAP *bmp)
{
   int unwrite_bank = (int)bmp->vtable->unwrite_bank;

   _asm {
      mov edx, bmp
      mov ecx, unwrite_bank
      call ecx
   }
}



/* fadd:
 *  Fixed point (16.16) addition.
 */
__inline fixed fadd(fixed x, fixed y)
{
   __asm {
      mov eax, x
      add eax, y
      jno Out1

      mov errno, ERANGE
      mov eax, 0x7FFFFFFF
      cmp y, 0
      jg Out1
      neg eax
 Out1:
   }
}



/* fsub:
 *  Fixed point (16.16) subtraction.
 */
__inline fixed fsub(fixed x, fixed y)
{
   _asm {
      mov eax, x
      sub eax, y
      jno Out1

      mov errno, ERANGE
      mov eax, 0x7FFFFFFF
      cmp y, 0
      jl Out1
      neg eax
 Out1:
   }
}



/* fmul:
 *  Fixed point (16.16) multiplication.
 */
__inline fixed fmul(fixed x, fixed y)
{
   _asm {
      mov eax, x
      imul y
      shrd eax, edx, 0x10
      shr edx, 0x10
      jz Out2
      cmp dx, 0xFFFF
      jz Out2
      mov errno, ERANGE
      mov eax, 0x7FFFFFFF
      cmp x, 0
      jge Out1
      neg eax
 Out1:
      cmp y, 0
      jge Out2
      neg eax
 Out2:
   }
}



/* fdiv:
 *  Fixed point (16.16) division.
 */
__inline fixed fdiv(fixed x, fixed y)
{
   _asm {
      mov ecx, y
      xor ebx, ebx
      mov eax, x
      or eax, eax
      jns Out1
      neg eax
      inc ebx
 Out1:
      or ecx, ecx
      jns Out2
      neg ecx
      inc ebx
 Out2:
      mov edx, eax
      shr edx, 0x10
      shl eax, 0x10
      cmp edx, ecx
      jae Out3
      div ecx
      or eax, eax
      jns Out4
 Out3:
      mov errno, ERANGE
      mov eax, 0x7FFFFFFF
 Out4:
      test ebx, 1
      je Out5
      neg eax
 Out5:
   }
}

