/*         ______   ___    ___ 
 *        /\  _  \ /\_ \  /\_ \ 
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *
 *      Configuration defines for use with Watcom.
 *
 *      By Shawn Hargreaves.
 *
 *      See readme.txt for copyright information.
 */


#ifndef __WATCOMC__
   #error bad include
#endif


#include <io.h>
#include <i86.h>
#include <conio.h>
#include <fcntl.h>
#include <direct.h>
#include <malloc.h>


#pragma disable_message (120 201)


/* describe this platform */
#define ALLEGRO_WATCOM
#define ALLEGRO_DOS
#define ALLEGRO_I386
#define ALLEGRO_LITTLE_ENDIAN
#define ALLEGRO_CONSOLE_OK
#define ALLEGRO_VRAM_SINGLE_SURFACE


/* describe how function prototypes look to Watcom */
#ifdef ALLEGRO_SRC

   #pragma aux default "_*" \
		     parm caller [] \
		     value struct float struct routine [eax] \
		     modify [eax ecx edx];

   #define AL_FUNC(type, name, args)            type name args
   #define AL_METHOD(type, name, args)          type (*name) args
   #define AL_FUNCPTR(type, name, args)         extern type (*name) args

#else

   #define AL_FUNC(type, name, args)            type __cdecl name args
   #define AL_METHOD(type, name, args)          type (__cdecl *name) args
   #define AL_FUNCPTR(type, name, args)         extern type (__cdecl *name) args

#endif

#ifndef AL_INLINE
   #define AL_INLINE(type, name, args, code)    AL_FUNC(type, name, args);
#endif


/* describe the asm syntax for this platform */
#define ALLEGRO_ASM_PREFIX    "_"
#define ALLEGRO_ASM_ALIGN     4
#define ALLEGRO_MMX


/* emulate some important djgpp routines */
#define inportb(port)         inp(port)
#define inportw(port)         inpw(port)
#define outportb(port, val)   outp(port, val)
#define outportw(port, val)   outpw(port, val)


typedef union __dpmi_regs
{
   struct {
      unsigned short di, di_hi, si, si_hi, bp, bp_hi, res, res_hi;
      unsigned short bx, bx_hi, dx, dx_hi, cx, cx_hi, ax, ax_hi;
      unsigned short flags, es, ds, fs, gs, ip, cs, sp, ss;
   } x;
} __dpmi_regs;


typedef struct __dpmi_paddr
{
  unsigned long offset32;
  unsigned short selector;
} __dpmi_paddr;


AL_FUNC(int, __dpmi_int, (int vector, __dpmi_regs *regs));


/* arrange for other headers to be included later on */
#define ALLEGRO_EXTRA_HEADER     "allegro/aldos.h"
#define ALLEGRO_INTERNAL_HEADER  "allegro/aintdos.h"

