#
#  Rules for building the Allegro library with RSXNT. This file is included
#  by the primary makefile, and should not be used directly.
#
#  This is not a full makefile: it uses the MSVC DLL, so it only has to
#  build an import library and the test/example programs. You must have
#  compiled the MSVC library before using this makefile.
#



# -------- define some variables that the primary makefile will use --------

PLATFORM = RSXNT
OBJ_DIR = obj/rsxnt/$(VERSION)
EXE = .exe
OBJ = .o

ifdef DEBUGMODE
LIB_BASENAME = libadimp.a
else
ifdef PROFILEMODE
LIB_BASENAME = libapimp.a
else
LIB_BASENAME = libalimp.a
endif
endif

LIB_NAME = lib/rsxnt/$(LIB_BASENAME)



# -------- check that the DJGPP environment variable is set --------

ifndef DJDIR
baddjgpp:
	@echo Your DJGPP environment variable is not set correctly!
endif

DJDIR_U = $(subst \,/,$(DJDIR))
DJDIR_D = $(subst /,\,$(DJDIR))



# -------- give a sensible default target for make without any args --------

.PHONY: _default

_default: default



# -------- decide what compiler options to use --------

ifdef WARNMODE
WFLAGS = -Wall -W -Werror -Wno-unused
else
WFLAGS = -Wall -Wno-unused
endif

ifdef PGCC
OFLAGS = -mpentium -O6 -ffast-math
else
OFLAGS = -m486 -O3 -ffast-math
endif

ifdef DEBUGMODE

# -------- debugging build --------
CFLAGS = -Zwin32 -DDEBUGMODE=$(DEBUGMODE) $(WFLAGS) -g
LFLAGS = -Zwin32 -g

else
ifdef PROFILEMODE

# -------- profiling build --------
CFLAGS = -Zwin32 $(WFLAGS) $(OFLAGS) -pg
LFLAGS = -Zwin32 -pg

else

# -------- optimised build --------
CFLAGS = -Zwin32 $(WFLAGS) $(OFLAGS) -fomit-frame-pointer
LFLAGS = -Zwin32

endif
endif



# -------- helper functions for cleaning and building the lib --------

CLEAN_FILES = obj/rsxnt/*/*.o lib/rsxnt/*

VERYCLEAN_FILES = obj/rsxnt/*/*.dep

define CLEAN
-rm -fv $(CLEAN_FILES)
endef

define VERYCLEAN
-rm -fv $(VERYCLEAN_FILES)
endef

$(LIB_NAME): lib/msvc/$(VERSION)$(LIBRARY_VERSION).dll

define MAKE_LIB
makelib lib/msvc/$(VERSION)$(LIBRARY_VERSION).dll -o $(LIB_NAME)
endef



# -------- rules for installing and removing the library files --------

$(DJDIR_U)/lib/$(LIB_BASENAME): $(LIB_NAME)
	copy lib\rsxnt\$(LIB_BASENAME) $(DJDIR_D)\lib
	@echo You should also run fixdjgpp and then make install that,
	@echo to get the documentation and headers copied across.

install: $(DJDIR_U)/lib/$(LIB_BASENAME)

uninstall:
	-rm -fv $(DJDIR_U)/lib/libalimp.a $(DJDIR_U)/lib/libadimp.a $(DJDIR_U)/lib/libapimp.a
	@echo All gone!



# -------- finally, we get to the fun part... --------

$(OBJ_DIR)/%.o: %.c
	gcc $(CFLAGS) -I. -I./include -o $@ -c $<

*/%.exe: $(OBJ_DIR)/%.o $(LIB_NAME)
	gcc $(LFLAGS) -o $@ $< $(LIB_NAME)



# -------- generate automatic dependencies --------

depend:
	gcc -Zwin32 -MM -MG -I. -I./include demo/*.c examples/*.c setup/*.c tests/*.c tools/*.c tools/plugins/*.c > _depend.tmp
	sed -e "s/^[a-zA-Z0-9_\/]*\///" _depend.tmp > _depend2.tmp
	sed -e "s/^\([a-zA-Z0-9_]*\)\.o:/obj\/rsxnt\/alleg\/\1\.o:/" _depend2.tmp > obj/rsxnt/alleg/makefile.dep
	sed -e "s/^\([a-zA-Z0-9_]*\)\.o:/obj\/rsxnt\/alld\/\1\.o:/" _depend2.tmp > obj/rsxnt/alld/makefile.dep
	sed -e "s/^\([a-zA-Z0-9_]*\)\.o:/obj\/rsxnt\/allp\/\1\.o:/" _depend2.tmp > obj/rsxnt/allp/makefile.dep
	rm _depend.tmp _depend2.tmp
