#
#  Rules for building the Allegro library with Watcom. This file is
#  included by the primary makefile, and should not be used directly.
#
#  This build uses djgpp for building the assembler sources and calculating
#  source dependencies, so you'll need to have that installed as well.
#
#  The "clean" and "veryclean" targets require an rm command to be
#  installed, eg. from the GNU fileutils package.
#
#  The "depend" target uses sed.
#



# -------- define some variables that the primary makefile will use --------

PLATFORM = Watcom
OBJ_DIR = obj/watcom/$(VERSION)
LIB_NAME = lib/watcom/$(VERSION).lib
RUNNER = obj/watcom/runner.exe
EXE = .exe
OBJ = .obj
HTML = htm



# -------- check that the WATCOM environment variable is set --------

.PHONY: badwatcom

ifndef WATCOM
badwatcom:
	@echo Your WATCOM environment variable is not set!
endif

WATDIR_U = $(subst \,/,$(WATCOM))
WATDIR_D = $(subst /,\,$(WATCOM))



# -------- give a sensible default target for make without any args --------

.PHONY: _default

_default: default



# -------- decide what compiler options to use --------

ifdef WARNMODE
WFLAGS = -w3 -we
else
WFLAGS = -w1
endif

OFLAGS = -ox -ot

ifdef DEBUGMODE

# -------- debugging build --------
CFLAGS = -DDEBUGMODE=$(DEBUGMODE) $(WFLAGS)
SFLAGS = -DDEBUGMODE=$(DEBUGMODE) -Wall

else
ifdef PROFILEMODE

# -------- profiling build --------
CFLAGS = $(WFLAGS) $(OFLAGS)
SFLAGS = -Wall

else

# -------- optimised build --------
CFLAGS = $(WFLAGS) $(OFLAGS)
SFLAGS = -Wall

endif
endif



# -------- list which platform specific objects to include --------

VPATH = src/dos src/i386 src/misc

OBJECT_LIST = $(COMMON_OBJECTS) $(I386_OBJECTS) modexgfx



# -------- helper functions for cleaning and building the lib --------

CLEAN_FILES = obj/watcom/*.exe obj/watcom/*.obj obj/watcom/*.inc \
	      obj/watcom/*/*.obj lib/watcom/*

VERYCLEAN_FILES = obj/watcom/*/*.dep

# note: the primary makefile appends more files to the above variables

define CLEAN
-rm -fv $(CLEAN_FILES)
endef

define VERYCLEAN
-rm -fv $(VERYCLEAN_FILES)
endef

define MAKE_LIB
$(RUNNER) $(subst /,\\,wlib -q -b -n $(LIB_NAME) $(addprefix +,$(OBJECTS)))
endef



# -------- rules for installing and removing the library files --------

$(WATDIR_U)/lib386/$(VERSION).lib: $(LIB_NAME)
	copy lib\watcom\$(VERSION).lib $(WATDIR_D)\lib386

$(WATDIR_U)/h/allegro.h: include/allegro.h
	copy include\allegro.h $(WATDIR_D)\h

$(WATDIR_U)/h/allegro:
	md $(WATDIR_D)\h\allegro

$(WATDIR_U)/h/allegro/%.h: include/allegro/%.h $(WATDIR_U)/h/allegro
	copy $(subst /,\,$<) $(subst /,\,$@)

HEADERS = $(addprefix $(WATDIR_U)/h/allegro/,$(notdir $(wildcard include/allegro/*.h)))

INSTALL_FILES = $(WATDIR_U)/lib386/$(VERSION).lib \
		$(WATDIR_U)/h/allegro.h \
		$(HEADERS)

install: $(INSTALL_FILES)

UNINSTALL_FILES = $(WATDIR_U)/lib386/alleg.lib \
		  $(WATDIR_U)/lib386/alld.lib \
		  $(WATDIR_U)/lib386/allp.lib \
		  $(WATDIR_U)/h/allegro.h \
		  $(WATDIR_U)/h/allegro/*.h

uninstall:
	-rm -fv $(UNINSTALL_FILES)
	-rd $(WATDIR_D)\h\allegro
	@echo All gone!



# -------- finally, we get to the fun part... --------

GCC2WATCOM = -D__WATCOMC__ -U__GNUC__ -UDJGPP

COMPILE_FLAGS = $(subst src/,-dALLEGRO_SRC ,$(findstring src/, $<))$(CFLAGS)

$(OBJ_DIR)/%.obj: %.c
	wcl386 $(subst /,\\,-zq -fr=nul $(COMPILE_FLAGS) -I. -I./include -fo=$@ -c $<)

$(OBJ_DIR)/%.obj: %.s
	gcc $(GCC2WATCOM) $(SFLAGS) -I. -I./include -x assembler-with-cpp -o $@ -c $<

*/%.exe: $(OBJ_DIR)/%.obj $(LIB_NAME)
	wcl386 $(subst /,\\,-zq -fr=nul -fe=$@ $< $(LIB_NAME))

obj/watcom/asmdef.inc: obj/watcom/asmdef.exe
	obj/watcom/asmdef.exe obj/watcom/asmdef.inc

obj/watcom/asmdef.exe: src/i386/asmdef.c include/*.h include/allegro/*.h
	wcl386 $(subst /,\\,-zq -fr=nul $(WFLAGS) -I. -I./include -fo=obj/watcom/asmdef.obj -fe=obj/watcom/asmdef.exe src/i386/asmdef.c)

obj/watcom/runner.exe: src/misc/runner.c
	gcc -Wall -Werror -s -o obj/watcom/runner.exe src/misc/runner.c



# -------- generate automatic dependencies --------

DEPEND_PARAMS = $(GCC2WATCOM) -MM -MG -I. -I./include \
		-nostdinc -isystem $(subst ;, -isystem ,$(INCLUDE))

depend:
	gcc $(DEPEND_PARAMS) src/*.c src/dos/*.c src/i386/*.c src/misc/*.c demo/*.c examples/*.c setup/*.c tests/*.c tools/*.c tools/plugins/*.c > _depend.tmp
	gcc $(DEPEND_PARAMS) -x assembler-with-cpp src/i386/*.s src/misc/*.s >> _depend.tmp
	sed -e "s/^[a-zA-Z0-9_\/]*\///" _depend.tmp > _depend2.tmp
	sed -e "s/^\([a-zA-Z0-9_]*\)\.o:/obj\/watcom\/alleg\/\1\.obj:/" _depend2.tmp > obj/watcom/alleg/makefile.dep
	sed -e "s/^\([a-zA-Z0-9_]*\)\.o:/obj\/watcom\/alld\/\1\.obj:/" _depend2.tmp > obj/watcom/alld/makefile.dep
	sed -e "s/^\([a-zA-Z0-9_]*\)\.o:/obj\/watcom\/allp\/\1\.obj:/" _depend2.tmp > obj/watcom/allp/makefile.dep
	rm _depend.tmp _depend2.tmp

