/*         ______   ___    ___ 
 *        /\  _  \ /\_ \  /\_ \ 
 *        \ \ \L\ \\//\ \ \//\ \      __     __   _ __   ___ 
 *         \ \  __ \ \ \ \  \ \ \   /'__`\ /'_ `\/\`'__\/ __`\
 *          \ \ \/\ \ \_\ \_ \_\ \_/\  __//\ \L\ \ \ \//\ \L\ \
 *           \ \_\ \_\/\____\/\____\ \____\ \____ \ \_\\ \____/
 *            \/_/\/_/\/____/\/____/\/____/\/___L\ \/_/ \/___/
 *                                           /\____/
 *                                           \_/__/
 *
 *      Main system driver for the Windows library.
 *
 *      By Shawn Hargreaves.
 *
 *      See readme.txt for copyright information.
 */


#include <string.h>
#include <ctype.h>

#include "allegro.h"
#include "allegro/aintern.h"
#include "allegro/aintwin.h"

#ifndef ALLEGRO_WINDOWS
   #error something is wrong with the makefile
#endif


static int sys_directx_init(void);
static void sys_directx_exit(void);
static void sys_directx_get_executable_name(char *output);
static void sys_directx_set_window_title(char *name);



/* the main system driver for running under DirectX */
SYSTEM_DRIVER system_directx =
{
   SYSTEM_DIRECTX,
   empty_string,
   empty_string,
   "DirectX",
   sys_directx_init,
   sys_directx_exit,
   sys_directx_get_executable_name,
   NULL,
   sys_directx_set_window_title,
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL,
   NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL
};



/* list of available system drivers */
_DRIVER_INFO _system_driver_list[] =
{
   {  SYSTEM_DIRECTX,   &system_directx,     TRUE  },
   {  0,                NULL,                0     }
};



/* sys_directx_init:
 *  Top level system driver wakeup call.
 */
static int sys_directx_init()
{
   /* todo */
   return 0;
}



/* sys_directx_exit:
 *  The end of the world...
 */
static void sys_directx_exit()
{
   /* todo */
}



/* sys_directx_get_executable_name:
 *  Return full path to the current executable.
 */
static void sys_directx_get_executable_name(char *output)
{
   char *cmd = GetCommandLine();
   int i = 0;
   int q;

   while ((cmd[i]) && (isspace(cmd[i])))
      i++;

   if ((cmd[i] == '\'') || (cmd[i] == '"'))
      q = cmd[i++];
   else
      q = 0;

   while ((cmd[i]) && ((q) ? (cmd[i] != q) : (!isspace(cmd[i]))))
      output += usetc(output, cmd[i++]);

   usetc(output, 0);
}



/* sys_directx_set_window_title:
 *  Alter the application title.
 */
static void sys_directx_set_window_title(char *name)
{
   /* todo */
}



/* _alWinMain:
 *  Entry point for Windows GUI programs, hooked by a macro in alwin.h,
 *  which makes it look as if the application can still have a normal
 *  main() function.
 */
int _WinMain(void *_main, int hInst, int hPrev, char *CmdLine, int nCmdShow)
{
   int (*mainfunc)(int argc, char *argv[]) = (int (*)(int, char *[]))_main;
   char argbuf[256];
   char *argv[64];
   int argc;
   int i, q;

   /* can't use parameter because it doesn't include the executable name */
   strcpy(argbuf, GetCommandLine());

   argc = 0;
   i = 0;

   /* parse commandline into argc/argv format */
   while (argbuf[i]) {
      while ((argbuf[i]) && (isspace(argbuf[i])))
	 i++;

      if (argbuf[i]) {
	 if ((argbuf[i] == '\'') || (argbuf[i] == '"')) {
	    q = argbuf[i++];
	    if (!argbuf[i])
	       break;
	 }
	 else
	    q = 0;

	 argv[argc++] = &argbuf[i];

	 while ((argbuf[i]) && ((q) ? (argbuf[i] != q) : (!isspace(argbuf[i]))))
	    i++;

	 if (argbuf[i]) {
	    argbuf[i] = 0;
	    i++;
	 }
      }
   }

   /* call the application entry point */
   return mainfunc(argc, argv);
}

